
// Copyright (c) 1996 Hukatronic (H.C.C.), Tomas Hurka. All Rights Reserved.

#import "Controller.h"

@implementation Controller

- awakeFromNib
{
	if (![dio1_popup isKindOf:[PopUpList class]]) dio1_popup = [dio1_popup target];
	if (![dio2_popup isKindOf:[PopUpList class]]) dio2_popup = [dio2_popup target];
	if (![dac_mode isKindOf:[PopUpList class]]) dac_mode = [dac_mode target];
	if (![adc_mode isKindOf:[PopUpList class]]) adc_mode = [adc_mode target];
	if (![dac_channel isKindOf:[PopUpList class]]) dac_channel = [dac_channel target];
	[dio1_popup setTarget:self];
	[dio2_popup setTarget:self];
	[dio1_popup setAction:@selector(popup:)];
	[dio2_popup setAction:@selector(popup:)];
	dio1=[[DAS_DIO alloc] initDIO:DIO1];
	dio2=[[DAS_DIO alloc] initDIO:DIO2];
	if (!dio1 || !dio2)
	{	NXRunAlertPanel("Error","Cannot connect to the DAS_12 driver","Exit",NULL,NULL);
		[NXApp terminate:self];
	}
	return self;
}

-popup:sender
{	const char *name;
	int tag;
	
	name=[[sender selectedCell] title];
	tag=[[sender selectedCell] tag];
	if ([sender window]==dio1_popup)
	{	[dio1_button setTitle:name];
		[dio1_button setTag:tag];
	}
	else
	{	[dio2_button setTitle:name];
		[dio2_button setTag:tag];
	}
	return self;
}

- run_dio:sender
{	id text;
	DAS_DIO *dio;
	
	if (sender==dio1_button)
	{	text=dio1_text;
		dio=dio1;
	}
	else
	{	text=dio2_text;
		dio=dio2;
	}
	if ([sender tag]==DIO_OUTPUT)
		[dio writeDigitalIO:[text intValue]];
	else
		[text setIntValue:[dio readDigitalIO]];
    return self;
}

- dac_extfreq:sender
{
	[dac_extfreq setEnabled:[sender state]];
	return self;
}

- check_dac:sender
{
	if ([dac isDACrunning])
	{	[self perform:@selector(check_dac:) with:sender afterDelay:60 cancelPrevious:YES];
	}
	else
	{	[sender setState:0];
		[sender setEnabled:YES];
		[dac free];
		dac=nil;
	}
	return self;
}

- run_dac:sender
{	int i,len=[dac_samples intValue];
	short *buf;
	BOOL zigzag;
	const char *item;
	int index=0;
	
	[sender setEnabled:NO];
	if (dac)
	{	NXRunAlertPanel("Internal error","dac variable must be nil!","Exit",NULL,NULL);
		[sender setState:0];
		[sender setEnabled:YES];
		return self;
	}
	if ([dac_extfreq isEnabled])
	{	dac=[[DAS_DAC alloc] initWithExternalDACSource:[dac_extfreq intValue]];
	}
	else
	{	dac=[[DAS_DAC alloc] init];
	}
	if (!dac)
	{	NXRunAlertPanel("Error","Cannot connect to the DAS_12 driver","Exit",NULL,NULL);
		[sender setState:0];
		[sender setEnabled:YES];
		return self;
	}
	buf=malloc(len*sizeof(*buf));
	item=[dac_mode selectedItem];
	if (item)
		index=[dac_mode indexOfItem:item];
	zigzag=index==1;
	for(i=0;i<len;i++)
	{	if (zigzag)
			buf[i]=i&0xFFF;
		else
			buf[i]=(i&1)?0xFFF:0x800;
	}
	index=0;
	item=[dac_channel selectedItem];
	if (item)
		index=[dac_channel indexOfItem:item];	
	[dac runDAC:index buffer:buf lenght:len repeat:1 frequency:[dac_freq intValue]];
	[self perform:@selector(check_dac:) with:sender afterDelay:60 cancelPrevious:YES];
	free(buf);
	return self;	
}

- run_adc:sender
{	int ch_start,ch_end, freq, len;
	int mode=0;
	const char *item;
	
	if (!(adc=[[DAS_ADC alloc] init]))
	{	NXRunAlertPanel("Error","Cannot connect to the DAS_12 driver","Exit",NULL,NULL);
		return self;
	}
	ch_start=[adc_start intValue];
	ch_end=[adc_end intValue];
	len=[adc_samples intValue];
	freq=[adc_freq intValue];
	item=[adc_mode selectedItem];
	if (item)
		mode=[adc_mode indexOfItem:item];	
	if (mode==2) mode=3;
	
	if (sound)
		[sound free];
	sound=[[DAS_Sample alloc] initWith:len freq:freq];

	[adc setScanMode:mode];
	[adc setChannels:ch_start:ch_end];
	[adc setTriggerSource:SOFTWARE];
	[adc setADCinternalPacingSource:freq];
	[adc runADC:(short *)[sound data] length:len];
	
//	free(buffer);
	[adc free];
	adc=nil;
	[adc_results setSound:sound];
	return self;
}

- free
{
	[dio1 free];
	[dio2 free];
	[dac free];
	[adc free];
	[sound free];
	return [super free];
}

- appWillTerminate:sender
{
	[self free];
	return self;
}

@end
