
// Copyright (c) 1996 Hukatronic (H.C.C.), Tomas Hurka. All Rights Reserved.

#import "DASBase.h"
#import "das_12.h"
#import "DAS_12_public.h"
#import <servers/netname.h>
#import <mach/mach_error.h>
#import <stdlib.h>
#import <appkit/nextstd.h>

#define DAS_12_DRIVER_ID "DAS_12"

@implementation DASBase
/*
	This is the basic class for all DAS_* objects. These object are designed to communicate with the DAS_12 driver. The DASBase handles connecting to the DAS_12 driver using mach ports.
*/

- init
/*
	connect to the DAS_12 driver server on localhost. Returns self if succeeded, nil otherwise
*/
{
	return [self initOnHost:""];
}

- initOnHost:(const char *)name
/*
	connect to the DAS_12 driver server on host name. Returns self if succeeded, nil otherwise
*/
{	kern_return_t   ret;    
	
	if (![super init])
	{	return nil;
	}
	 
	ret = netname_look_up(name_server_port, name, DAS_12_DRIVER_ID , &server);
	
	if (ret != KERN_SUCCESS)
	{
		NXLogError("%s - Lookup of das_12 server \"%s\"  on machine %s failed due to %s",
						[self name], DAS_12_DRIVER_ID,strlen(name)?name:"localhost",mach_error_string(ret));	
		[super free];
		return nil;
	}
	return self;
}

- free
/*
	This deallocates the server port and frees itself.
*/
{
	if (!server)
	{	kern_return_t   ret;
		if ((ret=port_deallocate(task_self(),server))!=KERN_SUCCESS)
		{	NXLogError("%s - port_deallocate failed due to %s", [self name],mach_error_string(ret));
		}
	}
	return [super free];
}

@end
