
// Copyright (c) 1996 Hukatronic (H.C.C.), Tomas Hurka. All Rights Reserved.

#import "DAS_DIO.h"
#import "das_12.h"
#import "DAS_12_public.h"
#import <appkit/nextstd.h>

@implementation DAS_DIO
/*
This class handles digital IO. The instance can be set to operate on DIO 0 or DIO 1. DIO 0 and DIO 1 are independet so you can have two instance of this class operating simulatenously. 
*/

extern port_t mig_get_reply_port();

- init
{
	NXLogError("%s - Use initDIO: or initDIO:onHost: instead of init to initialize instance of DAS_DIO class\n",[self name]);
	[super free];
	return nil;
}

- initDIO:(int)number
/*
Used to initialze DIO number. Number can be 0 or 1. This methods must be used instad of -init. To create DAS_DIO instalce for DIO 1 use [[DAS_DIO alloc] initDIO:DIO1]. Connect to the DAS_12 driver on the localhost. Returns self, if succeded, nil on error.
*/
{
	return [self initDIO:number onHost:""];
}

- initDIO:(int)number onHost:(const char *)name
/*
Used to initialze DIO number. Number can be 0 or 1. This methods must be used instad of -init. To create DAS_DIO instalce for DIO 1 use [[DAS_DIO alloc] initDIO:DIO1]. Connect to the DAS_12 driver on the host name. Returns self, if succeded, nil on error.
*/
{	kern_return_t ret;
	
	if (![super initOnHost:name])
		return nil;
	switch (number)
	{	case DIO1:
			ret=lockDIO1(server, mig_get_reply_port());
			break;
		case DIO2:
			ret=lockDIO2(server, mig_get_reply_port());
			break;
		default:
			NXLogError("%s - Bad DIO number %d\n",[self name],number);
			[super free];
			return nil;
	}
	
	if (ret!=KERN_SUCCESS)
	{	NXLogError("%s - DIO %d locked\n",[self name],number);
		[super free];
		return nil;
	}
	dio_number=number;
	setDigitalIO(server,number,DIO_INPUT);
	io_status=DIO_INPUT;
	return self;
}

- writeDigitalIO:(unsigned char)value
/* 
Writes value to the DIO. 
*/
{
	if (io_status!=DIO_OUTPUT)
	{	setDigitalIO(server,dio_number,DIO_OUTPUT);
		io_status=DIO_OUTPUT;
	}
	writeDigitalIO(server,dio_number,value);
	return self;
}

- (unsigned char)readDigitalIO
/*
Reads DIO. Returns value read from DIO.
*/
{	unsigned int v;

	if (io_status!=DIO_INPUT)
	{	setDigitalIO(server,dio_number,DIO_INPUT);
		io_status=DIO_INPUT;
	}
	readDigitalIO(server,dio_number,&v);
	return (unsigned char)v;
}

- (int)dio
/*
Returns the DIO number, on which the instance operates.
*/
{
	return dio_number;
}

- free
/*
Unlocks the DIO part of the driver. 
*/
{
	switch (dio_number)
	{	case DIO1:
			unlockDIO1(server);
			break;
		case DIO2:
			unlockDIO2(server);
			break;
		default:
			NXLogError("%s - Bad DIO number %d\n",[self name],dio_number);
			return [super free];
	}
	return [super free];
}
@end
